#!/usr/bin/env python3
"""
memo-text - ColdSync conduit for syncing Memo Pad with plain text files.

The file layout this produces is Unfiled memos in the top level, and subfolders
named after each other category. Filenames are made from the first line plus an
ID in hex; when creating a new text file you can name it whatever you want and
it'll be rewritten on first sync.

Caveats:
* Due to how Palm categories work, you can't have more than 15 subfolders, or
  more than one level deep.
* Will likely mess up manual sorting in Memo Pad.

Required headers:
* Where: <path to memos folder>
"""
from coldsync import conduit_main
from itertools import chain
from palm import MemoDB
from pathlib import Path
import re

MAX_MEMO_SIZE = 4096

def _text_to_filename(data):
    """Get the first few (valid filename) characters of the first line"""
    if '\n' in data:
        data = data[:data.index('\n')]
    data = re.sub(r'[^a-zA-Z0-9 _.\-]', '', data)
    data = data.lstrip()
    return data[:13].rstrip()

def text_to_pdb(_, out_fname, prefs, Where, **headers):
    if not Path(out_fname).is_file():
        raise NotImplementedError("memo-text can't create the PDB from "
                                  "scratch, try again")
    with open(out_fname, 'rb') as f:
        db = MemoDB(f)
    # Get existing categories
    #TODO
    with open(out_fname, 'wb') as f:
        db.save(f)

def pdb_to_text(in_fname, _, prefs, Where, Delete='no', **headers):
    with open(in_fname, 'rb') as f:
        db = MemoDB(f)
    print('004 Dumping memos...')
    base_dir = Path(Where)
    delete = Delete == 'yes'
    seen = set()
    for i, r in enumerate(db.recordList):
        cat_id = r.attributes.categoryID
        cat_name = db.category_name(cat_id)
        new_fname = '{}{}-{:06x}.txt'.format(
                '.' if r.attributes.Secret else '',
                _text_to_filename(db.recordData[i]), r.uniqueID)
        if cat_id > 0:
            new_path = base_dir / cat_name / new_fname
            new_path.parent.mkdir(exist_ok=True)
        else: # Unfiled goes in root
            new_path = base_dir / new_fname
        search = '*-{:06x}.txt'.format(r.uniqueID)
        try:
            old_path = next(chain(base_dir.glob('*/'+search), base_dir.glob(search)))
            if old_path != new_path:
                print('004 rename {} to {}'.format(old_path, new_path))
                old_path.rename(new_path)
        except StopIteration:
            old_path = None
        print('004 {}'.format(new_fname))
        if r.attributes.Busy:
            print('004 busy')
        if r.attributes.Dirty:
            print('004 dirty')
        if r.attributes.Delete:
            print('004 delete')
        #TODO check if record has changed before rewriting
        with new_path.open('w') as f:
            f.write(db.recordData[i])
        seen.add(new_path)
    for p in chain(base_dir.glob('*/*.txt'), base_dir.glob('*.txt')):
        if p not in seen:
            if p.stat().st_size > MAX_MEMO_SIZE:
                print('004 Ignoring too-large file {}'.format(p))
                continue
            if delete:
                p.unlink()
            else:
                print('004 Not deleting {} (set Delete:yes to delete it)'.format(p))

if __name__ == '__main__':
    conduit_main(fetch=text_to_pdb, dump=pdb_to_text)
